function Flagfox_handleClick(event)
{
    switch (event.button)
    {
        case 0:
            if (!event.ctrlKey)
            {
                Flagfox_handleAction("Geotool");
                return;
            } // else fallthrough; ctrl+click == middle-click
        case 1:
            Flagfox_handleAction(FFpreferences.getCharPref("flagfox.middleclick"));  // Set action via options
            return;
        // Button 2 shows popup menu via context attribute
    }
}

function Flagfox_handleCursor(imagenode)  // Changes mouseover cursor to a hand when there is a click action
{
    imagenode.style.cursor = Flagfox_isActionAllowed("Geotool") ? "pointer" : "default" ;
}

function Flagfox_handleMenu(menupopup)  // Decides which menu items to grey out
{
    var menuItems = menupopup.getElementsByTagName("menuitem");
    for (var i=0; i < menuItems.length; i++)  // menuItems[i].value is not reliable in FF3betas; using getAttribute() instead!
        menuItems[i].setAttribute( "disabled", !Flagfox_isActionAllowed(menuItems[i].getAttribute("value")) );
}

function Flagfox_isActionAllowed(action)
{
    function isPrivateIP()
    {
        return flagState.country != null && flagState.country[0][0] == '-';  // -A, -B, -C, -L
    }

    switch (action)
    {
        case "Geotool":   return flagState.ip != "" && !isPrivateIP();    // Needs a non-private IP
        case "Wikipedia": return flagState.country != null;               // Needs a fully looked-up location
        case "Whois":     return flagState.host != "" && !isPrivateIP();  // Just needs a host, but can't be a private IP
        case "CopyIP":    return flagState.ip != "";                      // Just needs any IP

        case "Options":   return true;
        case "Nothing":   return false;  // Set "flagfox.middleclick" to this to disable that feature

        default:
            Flagfox_error(true,"Invalid Flagfox action: "+action,null);
            return false;
    }
}

function Flagfox_handleAction(action)
{
    if (!Flagfox_isActionAllowed(action))
        return;  // Disallowed or invalid action

    switch (action)
    {
        case "Geotool":
            // Geotool figures out the locale from the HTTP header
            // We send ip and host in case Geotool resolves to a different server, based on its different location
            Flagfox_openLink("http://geotool.servehttp.com/?ip=" + flagState.ip + "&host=" + flagState.host);
            return;

        case "Wikipedia":
            var wiki;
            switch (flagState.country[0])
            {
                case '-A':  case '-B':  case '-C':
                    wiki = "RFC 1918";
                    break;
                default:  // For 'A1', 'A2', and '-L' flagState.country[1] == translated range description
                    wiki = encodeURIComponent(flagState.country[1]);
                    break;
            }
            var language = Components.classes["@mozilla.org/network/protocol;1?name=http"]
                                     .getService(Components.interfaces.nsIHttpProtocolHandler)
                                     .language.split('-')[0];  // language-dialect --> language
            // We do a search here, just in case the exact page doesn't exist or there is no localized version
            Flagfox_openLink("http://" + language + ".wikipedia.org/wiki/Special:Search?search=" + wiki + "&go=Go");
            return;

        case "Whois":
            // DomainTools can parse TLD correctly, as well as full domain names
            Flagfox_openLink("http://whois.domaintools.com/" + flagState.host);
            return;

        case "CopyIP":
            const clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
                                              .getService(Components.interfaces.nsIClipboardHelper);
            clipboardHelper.copyString(flagState.ip);
            return;

        case "Options":
            window.openDialog("chrome://flagfox/content/options.xul",
                              "FlagfoxOptions", "chrome,dialog,centerscreen");
            return;
    }
}

function Flagfox_openLink(url)
{
    try
    {
        var tabPrefs = FFpreferences.getCharPref("flagfox.openlinksin");

        if (tabPrefs == "tabBG" || tabPrefs == "tabFG")
        {
            var newTab = window.getBrowser().addTab(url,null,null);
            if (tabPrefs == "tabFG")
                window.getBrowser().selectedTab = newTab;
        }
        else if (tabPrefs == "currentTab")
        {
            window.content.document.location = url;
        }
        else  // "winBG" || "winFG"
        {
            var newWindow = window.open(url,"_blank");
            if (tabPrefs == "winBG")
            {
                newWindow.blur();
                window.focus();
            }
        }
    } catch (e) { Flagfox_error(true,"Failed to open URL: "+url,e); }
}
